@@ -10,13 +10,18 @@ import Crypto.PubKey.RSA.PKCS15 as PKCS15
 import Crypto.PubKey.RSA.OAEP as OAEP
 import Crypto.PubKey.RSA.PSS as PSS
 import Crypto.PubKey.HashDescr
+import Crypto.PubKey.ECC.ECDSA as ECDSA
 import Crypto.Random
+import Control.DeepSeq
 
 import qualified Data.ByteString as B
 
 right (Right r) = r
 right (Left _)  = error "left received"
 
+instance NFData Signature where
+    rnf (Signature r s) = rnf r `seq` rnf s
+
 main = do
     rng <- cprgCreate `fmap` createEntropyPool :: IO SystemRNG
     let !bs = B.replicate 32 0
@@ -28,6 +33,8 @@ main = do
         !blinder = fst $ generateBlinder rng (RSA.public_n rsaPublickey)
         oaepParams = OAEP.defaultOAEPParams SHA1.hash
         pssParams  = PSS.defaultPSSParamsSHA1
+        ecdsaSignatureP = fst $ ECDSA.sign rng ecdsaPrivatekeyP SHA1.hash bs
+        ecdsaSignatureB = fst $ ECDSA.sign rng ecdsaPrivatekeyB SHA1.hash bs
     defaultMain
         [ bgroup "RSA PKCS15"
             [ bench "encryption" $ nf (right . fst . PKCS15.encrypt rng rsaPublickey) bs
@@ -63,4 +70,12 @@ main = do
                 ]
             , bench "verify" $ nf (PSS.verify pssParams rsaPublickey bs) signedMsgPSS
             ]
+        , bgroup "ECDSA secp160r1"
+            [ bench "sign" $ nf (fst . ECDSA.sign rng ecdsaPrivatekeyP SHA1.hash) bs
+            , bench "verify"  $ nf (ECDSA.verify SHA1.hash ecdsaPublickeyP ecdsaSignatureP) bs
+            ]
+        , bgroup "ECDSA sect163k1"
+            [ bench "sign" $ nf (fst . ECDSA.sign rng ecdsaPrivatekeyB SHA1.hash) bs
+            , bench "verify"  $ nf (ECDSA.verify SHA1.hash ecdsaPublickeyB ecdsaSignatureB) bs
+            ]
         ]
